#!/usr/bin/env python3

import amulet
import requests
import sys
import time
import traceback
from pymongo import MongoClient


class BasicMongo(object):
    def __init__(self, units, series, deploy_timeout):
        self.units = units
        self.series = series
        self.deploy_timeout = deploy_timeout
        self.d = amulet.Deployment(series=self.series)
        self.addy = None

    def deploy(self):
        try:
            self.d.setup(self.deploy_timeout)
            self.d.sentry.wait(self.deploy_timeout)
        except amulet.helpers.TimeoutError:
            message = 'The environment did not setup in %d seconds.',
            self.deploy_timeout
            amulet.raise_status(amulet.SKIP, msg=message)
        except:
            raise

        self.sentry_dict = {svc: self.d.sentry[svc]
                            for svc in list(self.d.sentry.unit)}

    def validate_status_interface(self):
        addy = self.addy
        fmt = "http://{}:28017"
        if ":" in addy:
            fmt = "http://[{}]:28017"

        time_between = 10
        tries = self.deploy_timeout / time_between

        try:
            r = requests.get(fmt.format(addy), verify=False)
            r.raise_for_status()
        except requests.exception.ConnectionError as ex:
            sys.stderr.write(
                'Connection error, sleep and retry... to {}: {}\n'.
                format(addy, ex))
            tb_lines = traceback.format_exception(ex.__class__,
                                                  ex, ex.__traceback__)
            tb_text = ''.join(tb_lines)
            sys.stderr.write(tb_text)
            tries = tries - 1
            if tries < 0:
                sys.stderr.write('retry limit caught, failing...\n')
            time.sleep(time_between)

    def validate_world_connectivity(self):
        addy = self.addy
        # ipv6 proper formating
        if ":" in addy:
            addy = "[{}]".format(addy)

        client = MongoClient(addy)
        db = client['test']

        # Can we successfully insert?
        insert_id = db.amulet.insert({'assert': True})
        if insert_id is None:
            amulet.raise_status(amulet.FAIL, msg="Failed to insert test data")

        # Can we delete from a shard using the Mongos hub?
        result = db.amulet.remove(insert_id)
        if 'err' in result and result['err'] is not None:
            amulet.raise_status(amulet.FAIL, msg="Failed to remove test data")

    def validate_running_services(self):
        for service in self.sentry_dict:
            grep_command = 'grep RELEASE /etc/lsb-release'
            release = self.sentry_dict[service].run(grep_command)
            release = str(release).split('=')[1]
            if release >= '15.10':
                status_string = 'active (running)'  # systemd
            else:
                status_string = 'mongodb start/running'  # upstart

            output = self.sentry_dict[service].run('service mongodb status')
            service_active = str(output).find(status_string)
            if service_active == -1:
                message = "Failed to find running MongoDB on host {}".format(
                    service)
                amulet.raise_status(amulet.SKIP, msg=message)
