#!/usr/bin/env python3

import amulet
import sys
import time
import traceback
from pymongo import MongoClient
from pymongo.errors import OperationFailure
from collections import Counter

from base_deploy import BasicMongo

# max amount of time to wait before testing for replicaset status
wait_for_replicaset = 600


class Replicaset(BasicMongo):
    def __init__(self, series):
        super(Replicaset, self).__init__(units=3,
                                         series=series,
                                         deploy_timeout=1800)

    def _expect_replicaset_counts(self,
                                  primaries_count,
                                  secondaries_count,
                                  time_between=10):
        unit_status = []
        tries = wait_for_replicaset / time_between

        for service in self.sentry_dict:
            addy = self.sentry_dict[service].info['public-address']
            if ":" in addy:
                addy = "[{}]".format(addy)
            while True:
                try:
                    client = MongoClient(addy)
                    r = client.admin.command('replSetGetStatus')
                    break
                except OperationFailure as ex:
                    sys.stderr.write(
                        'OperationFailure, sleep and retry... to {}: {}\n'.
                        format(addy, ex))
                    tb_lines = traceback.format_exception(ex.__class__,
                                                          ex, ex.__traceback__)
                    tb_text = ''.join(tb_lines)
                    sys.stderr.write(tb_text)
                    tries = tries - 1
                    if tries < 0:
                        sys.stderr.write('retry limit caught, failing...\n')
                        break
                    time.sleep(time_between)
            unit_status.append(r['myState'])
            client.close()

        primaries = Counter(unit_status)[1]
        if primaries != primaries_count:
            message = "Expected %d PRIMARY unit(s)! Found: %s %s" % (
                primaries_count,
                primaries,
                unit_status)
            amulet.raise_status(amulet.FAIL, message)

        secondrs = Counter(unit_status)[2]
        if secondrs != secondaries_count:
            message = ("Expected %d secondary units! (Found %s) %s" %
                       (secondaries_count, secondrs, unit_status))
            amulet.raise_status(amulet.FAIL, message)

    def deploy(self):
        self.d.add('mongodb', charm='mongodb', units=self.units)
        self.d.expose('mongodb')
        super(Replicaset, self).deploy()
        self.wait_for_replicaset = 600

    def validate_status_interface(self):
        self.addy = self.d.sentry['mongodb'][0].info['public-address']
        super(Replicaset, self).validate_status_interface()

    def validate_replicaset_setup(self):
        self.d.sentry.wait(self.deploy_timeout)
        self._expect_replicaset_counts(1, 2)

    def validate_replicaset_relation_joined(self):
        self.d.add_unit('mongodb', units=2)
        self.d.sentry.wait(wait_for_replicaset)
        self.sentry_dict = {svc: self.d.sentry[svc]
                            for svc in list(self.d.sentry.unit)}
        self._expect_replicaset_counts(1, 4)

    def validate_world_connectivity(self):
        # figuring out which unit is primary
        primary = False
        while not primary:
            for unit in self.sentry_dict:
                unit_address = self.sentry_dict[unit].info['public-address']
                if ":" in unit_address:
                    unit_address = "[{}]".format(unit_address)
                c = MongoClient(unit_address)
                r = c.admin.command('replSetGetStatus')
                if r['myState'] == 1:
                    # reusing address without possible brackets []
                    primary = self.sentry_dict[unit].info['public-address']
                    break
                time.sleep(.1)

        self.addy = primary
        super(Replicaset, self).validate_world_connectivity()

    def validate_running_services(self):
        super(Replicaset, self).validate_running_services()

    def run(self):
        self.deploy()
        self.validate_status_interface()
        self.validate_running_services()
        self.validate_replicaset_setup()
        self.validate_replicaset_relation_joined()
        self.validate_world_connectivity()
