#!/usr/bin/env python3

from base_deploy import BasicMongo

import amulet
from pymongo import MongoClient
from collections import Counter


class WithStorage(BasicMongo):
    def __init__(self, series):
        super(WithStorage, self).__init__(units=2,
                                          series=series,
                                          deploy_timeout=1800)

    def deploy(self):
        self.d.add('mongodb',
                   charm='mongodb',
                   units=self.units,
                   constraints={'root-disk': '20480M'})

        storage_charm = 'cs:~chris-gondolin/{}/storage-5'.format(self.series)
        self.d.add('storage', charm=storage_charm, series=self.series)
        self.d.configure('storage', {'provider': 'local'})
        super(WithStorage, self).deploy()
        self.d.expose('mongodb')

        ordered_units = sorted(self.d.sentry['mongodb'],
                               key=lambda u: u.info['unit'])
        self.sentry_dict = {
                            'mongodb0-sentry': ordered_units[0],
                            'mongodb1-sentry': ordered_units[1]
                            }

    def validate_status(self):
        self.d.sentry.wait_for_status(self.d.juju_env, ['mongodb'])

    def validate_replicaset_setup(self):
        self.d.sentry.wait(self.deploy_timeout)

        unit_status = []
        for service in self.sentry_dict:
            addy = self.sentry_dict[service].info['public-address']
            if ":" in addy:
                addy = "[{}]".format(addy)
            client = MongoClient(addy)
            r = client.admin.command('replSetGetStatus')
            unit_status.append(r['myState'])
            client.close()

        prims = Counter(unit_status)[1]
        if prims != 1:
            message = "Only one PRIMARY unit allowed! Found: %s" % (prims)
            amulet.raise_status(amulet.FAIL, message)

        secnds = Counter(unit_status)[2]
        if secnds != 1:
            message = "Only one SECONDARY unit allowed! (Found %s)" % (secnds)
            amulet.raise_status(amulet.FAIL, message)

    def run(self):
        self.deploy()
        self.validate_status()
        self.validate_replicaset_setup()

        print("Adding storage relation, and sleeping for 2 min.")
        try:
            self.d.relate('mongodb:data', 'storage:data')
        except OSError as e:
            print("Ignoring error: {}", e)
        self.d.sentry.wait(120)       # 2 minute

        self.validate_status()
        self.validate_replicaset_setup()
